@@ -1,33 +1,40 @@
-# $Id: renderer.py 72631 2008-09-28 15:53:58Z thomasw $
+# $Id: renderer.py 83100 2009-03-24 19:50:09Z jensens $
 
 from AccessControl import getSecurityManager, Unauthorized
 from Globals import DTMLFile
 
-from zope.interface import directlyProvidedBy, directlyProvides
+from zope.interface import directlyProvides
+from zope.component import queryMultiAdapter
 from zope.component import getMultiAdapter
 
 from Products.Five.browser import BrowserView
 
-from Products.Collage.interfaces import ICollageBrowserLayer, IDynamicViewManager
+from Products.Collage.interfaces import IDynamicViewManager
 from Products.Collage.interfaces import ICollageAlias
-
 from Products.Collage.utilities import isTranslatable
+from Products.Collage.viewmanager import mark_request
 
 class SimpleContainerRenderer(BrowserView):
+    
     def getItems(self, contents=None):
+        """Items are a views to render.
+        
+        @param contents: If given fetch the folderListingFolderContents of 
+                         context.
+        @return: a list of views ready to render.
+        """
         # needed to circumvent bug :-(
         self.request.debug = False
 
         # transmute request interfaces
-        ifaces = directlyProvidedBy(self.request)
-        directlyProvides(self.request, ICollageBrowserLayer)
+        ifaces = mark_request(self.request)
 
         views = []
-
-        if not contents:
+        if contents is None:
             contents = self.context.folderlistingFolderContents()
-
         for context in contents:
+            if context is None:
+                continue
             target = context
             manager = IDynamicViewManager(context)
             layout = manager.getLayout()
@@ -66,8 +73,12 @@ class SimpleContainerRenderer(BrowserView):
                     canmanager = IDynamicViewManager(target.getCanonical())
                     layout = manager.getLayout() or canmanager.getLayout() or layout
 
-            # assume that a layout is always available
-            view = getMultiAdapter((target, self.request), name=layout)
+            # don't assume that a layout is always available
+            view = queryMultiAdapter((target, self.request), name=layout)            
+            if view is None:
+                view = getMultiAdapter((target, self.request), 
+                                         name='error_collage-view-not-found')
+                view.notfoundlayoutname = layout            
 
             # store reference to alias if applicable
             if ICollageAlias.providedBy(context):
@@ -77,7 +88,6 @@ class SimpleContainerRenderer(BrowserView):
 
         # restore interfaces
         directlyProvides(self.request, ifaces)
-
         return views
 
 class CollageStylesheet(BrowserView):
